from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.publisher.browser import TestRequest
from ZPublisher.HTTPResponse import HTTPResponse

import os.path
import unittest


class DAVTestRequest(TestRequest):
    """Mock webdav request."""

    get_header = TestRequest.getHeader

    def _createResponse(self):
        return HTTPResponse()


class WebDAVIntegrationTest(unittest.TestCase):
    """Test webdav support."""

    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Image", "image")
        self.image = self.portal["image"]
        self.image.title = "My Image"
        self.portal.invokeFactory("File", "file")
        self.file = self.portal["file"]
        self.file.title = "My file"

    def test_image_put(self):
        """Upload an image through webdav."""
        filename = os.path.join(os.path.dirname(__file__), "image.jpg")
        with open(filename, "rb") as myfile:
            request = DAVTestRequest(
                environ={
                    "BODYFILE": myfile,
                    "PATH_INFO": "/foo/bar/image.jpg",
                }
            )
            self.image.REQUEST = request
            self.image.PUT()
        self.assertEqual(self.image.image.filename, "image.jpg")
        self.assertEqual(self.image.get_size(), 5131)
        self.assertEqual(self.image.content_type(), "image/jpeg")

    def test_file_put(self):
        """Upload a file through webdav."""
        filename = os.path.join(os.path.dirname(__file__), "file.pdf")
        with open(filename, "rb") as myfile:
            request = DAVTestRequest(
                environ={
                    "BODYFILE": myfile,
                    "PATH_INFO": "/foo/bar/file.pdf",
                }
            )
            self.file.REQUEST = request
            self.file.PUT()
        self.assertEqual(self.file.file.filename, "file.pdf")
        self.assertEqual(self.file.get_size(), 8561)
        self.assertEqual(self.file.content_type(), "application/pdf")



from Acquisition import aq_inner
from DateTime import DateTime
from plone.app.contenttypes.behaviors.collection import (
    ICollection as ICollection_behavior,
)
from plone.app.contenttypes.interfaces import ICollection
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING
from plone.app.contenttypes.testing import set_browserlayer
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.textfield.value import RichTextValue
from plone.dexterity.interfaces import IDexterityFTI
from plone.testing.zope import Browser
from transaction import commit
from zope.component import createObject
from zope.component import queryUtility
from zope.interface import alsoProvides

import os.path
import unittest


query = [
    {
        "i": "Title",
        "o": "plone.app.querystring.operation.string.contains",
        "v": "Collection Test Page",
    }
]


def dummy_image():
    from plone.namedfile.file import NamedBlobImage

    filename = os.path.join(os.path.dirname(__file__), "image.png")
    with open(filename, "rb") as f:
        image_data = f.read()
    return NamedBlobImage(data=image_data, filename=filename)


class PloneAppCollectionClassTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Collection", "collection")
        self.collection = self.portal["collection"]

    def test_syndicatable(self):
        from plone.base.interfaces.syndication import ISyndicatable

        self.assertTrue(ISyndicatable.providedBy(self.collection))


class PloneAppCollectionIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Folder", "test-folder")
        self.folder = self.portal["test-folder"]

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="Collection")
        schema = fti.lookupSchema()
        self.assertTrue(schema.getName().endswith("_0_Collection"))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="Collection")
        self.assertNotEqual(None, fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="Collection")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(ICollection.providedBy(new_object))

    def test_adding(self):
        self.folder.invokeFactory("Collection", "collection1")
        p1 = self.folder["collection1"]
        self.assertTrue(ICollection.providedBy(p1))


class PloneAppCollectionViewsIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer["app"])
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        set_browserlayer(self.request)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Folder", "test-folder")
        self.folder = self.portal["test-folder"]
        self.folder.invokeFactory("Collection", "collection1")
        self.collection = aq_inner(self.folder["collection1"])
        self.request.set("URL", self.collection.absolute_url())
        self.request.set("ACTUAL_URL", self.collection.absolute_url())

    def test_collection_view(self):
        view = self.collection.restrictedTraverse("@@view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)

    def test_collection_listing_view(self):
        view = self.collection.restrictedTraverse("listing_view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)

    def test_collection_summary_view(self):
        view = self.collection.restrictedTraverse("summary_view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)

    def test_collection_full_view(self):
        view = self.collection.restrictedTraverse("full_view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)

    def test_collection_tabular_view(self):
        view = self.collection.restrictedTraverse("tabular_view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)

    def test_collection_album_view(self):
        view = self.collection.restrictedTraverse("album_view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)

    def test_add_collection(self):
        browser = self.browser
        browser.handleErrors = False
        browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )
        portal_url = self.portal.absolute_url()
        browser.open(portal_url)
        browser.getLink(url="http://nohost/plone/++add++Collection").click()
        widget = "form.widgets.IDublinCore.title"
        browser.getControl(name=widget).value = "My collection"
        widget = "form.widgets.IDublinCore.description"
        browser.getControl(name=widget).value = "This is my collection."
        widget = "form.widgets.IRichTextBehavior.text"
        browser.getControl(name=widget).value = "Lorem Ipsum"
        widget = "form.widgets.IShortName.id"
        browser.getControl(name=widget).value = "my-special-collection"
        browser.getControl("Save").click()
        self.assertTrue(browser.url.endswith("my-special-collection/view"))
        self.assertTrue("My collection" in browser.contents)
        self.assertTrue("This is my collection" in browser.contents)
        self.assertTrue("Lorem Ipsum" in browser.contents)

    def test_collection_templates(self):
        self.portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ["Manager"], []
        )
        browser = self.browser
        portal = self.portal
        login(portal, SITE_OWNER_NAME)
        # add an image that will be listed by the collection
        portal.invokeFactory("Image", "image", title="Image example")

        image = self.portal["image"]
        image.image = dummy_image()

        # add a collection, so we can add a query to it
        portal.invokeFactory("Collection", "collection", title="New Collection")
        collection = portal["collection"]
        # Search for images
        query = [
            {
                "i": "Type",
                "o": "plone.app.querystring.operation.string.is",
                "v": "Image",
            }
        ]
        collection.text = RichTextValue(
            "Lorem collection ipsum", "text/plain", "text/html"
        )

        wrapped = ICollection_behavior(collection)
        # set the query and publish the collection
        wrapped.query = query
        workflow = portal.portal_workflow
        workflow.doActionFor(collection, "publish")
        commit()
        logout()
        # open a browser to see if our image is in the results
        browser.handleErrors = False
        url = collection.absolute_url()
        browser.open(url)
        self.assertIn("Lorem collection ipsum", browser.contents)
        self.assertIn("Image example", browser.contents)

        # open summary_view template
        browser.open(f"{url}/@@summary_view")
        self.assertIn("Lorem collection ipsum", browser.contents)
        self.assertIn("Image example", browser.contents)

        # open full_view template
        browser.open(f"{url}/@@full_view")
        self.assertIn("Lorem collection ipsum", browser.contents)
        self.assertIn("Image example", browser.contents)

        # open tabular_view template
        browser.open(f"{url}/@@tabular_view")
        self.assertIn("Lorem collection ipsum", browser.contents)
        self.assertIn("Image example", browser.contents)

        # open thumbnail_view template
        browser.open(f"{url}/@@album_view")
        self.assertIn("Lorem collection ipsum", browser.contents)
        self.assertIn("Image example", browser.contents)

    def test_sorting_1(self):
        self.portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ["Manager"], []
        )

        portal = self.portal
        login(portal, SITE_OWNER_NAME)
        query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.string.is",
                "v": "News Item",
            }
        ]
        portal.invokeFactory(
            "Collection",
            "collection",
            title="New Collection",
            query=query,
            sort_on="created",
            sort_reversed=True,
        )

        now = DateTime()
        # News Item 1
        portal.invokeFactory(id="newsitem1", type_name="News Item")
        item1 = portal.newsitem1
        item1.creation_date = now - 2
        item1.reindexObject()
        # News Item 2
        portal.invokeFactory(id="newsitem2", type_name="News Item")
        item2 = portal.newsitem2
        item2.creation_date = now - 1
        item2.reindexObject()
        # News Item 3
        portal.invokeFactory(id="newsitem3", type_name="News Item")
        item3 = portal.newsitem3
        item3.creation_date = now
        item3.reindexObject()

        collection = portal["collection"]
        wrapped = ICollection_behavior(collection)
        results = wrapped.results(batch=False)
        ritem0 = results[0]
        ritem1 = results[1]
        ritem2 = results[2]

        self.assertTrue(ritem0.CreationDate() > ritem1.CreationDate())
        self.assertTrue(ritem1.CreationDate() > ritem2.CreationDate())

    def test_custom_query(self):
        self.portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ["Manager"], []
        )
        portal = self.portal
        login(portal, SITE_OWNER_NAME)
        query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.string.is",
                "v": ["News Item", "Document"],
            }
        ]
        portal.invokeFactory(
            "Collection",
            "collection",
            title="New Collection",
            query=query,
        )

        # item 1
        portal.invokeFactory(id="testnews", type_name="News Item")
        item1 = portal.testnews
        item1.reindexObject()

        # item 2
        portal.invokeFactory(id="testdoc", type_name="Document")
        item2 = portal.testdoc
        item2.reindexObject()

        collection = portal["collection"]
        wrapped = ICollection_behavior(collection)

        # Test unmodified query
        results = wrapped.results(batch=False)
        self.assertEqual(len(results), 2)

        # Test with custom query
        results = wrapped.results(batch=False, custom_query={"portal_type": "Document"})
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0].id, "testdoc")

        # Test with custom query, which should not find anything
        results = wrapped.results(
            batch=False, custom_query={"portal_type": "Document", "id": "bla"}
        )
        self.assertEqual(len(results), 0)

    def test_respect_navigation_root(self):
        self.portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ["Manager"], []
        )
        portal = self.portal
        login(portal, SITE_OWNER_NAME)

        # Create two subsites i.e create two folders and mark them with
        # INavigationRoot
        for i in range(1, 3):
            folder_id = f"folder{i}"
            portal.invokeFactory("Folder", folder_id, title=f"Folder{i}")
            folder = portal[folder_id]
            alsoProvides(folder, INavigationRoot)
        folders = (portal["folder1"], portal["folder2"])

        # Add a content item to each folder
        for f in folders:
            f_id = f.getId()
            f.invokeFactory("Document", f"item_in_{f_id}", title=f"Item In {f_id}")

        # Add a collection to folder1
        folder1 = folders[0]
        folder1.invokeFactory("Collection", "collection1", title="Collection 1")
        collection1 = folder1["collection1"]
        wrapped = ICollection_behavior(collection1)
        wrapped.query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.string.is",
                "v": "Document",
            },
            # use a "/" path and navroot works fine!
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.path",
                "v": "/",
            },
        ]

        # Check if only the item inside folder1 is returned, since it's a
        # navigation root.
        items = wrapped.results(batch=False)
        ids = [i.getId() for i in items]
        self.assertListEqual(ids, ["item_in_folder1"])


class PloneAppCollectionEditViewsIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        set_browserlayer(self.request)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Folder", "test-folder")
        self.folder = self.portal["test-folder"]
        self.folder.invokeFactory("Collection", "collection1")
        self.collection = aq_inner(self.folder["collection1"])
        self.request.set("URL", self.collection.absolute_url())
        self.request.set("ACTUAL_URL", self.collection.absolute_url())

    def test_search_result(self):
        view = self.collection.restrictedTraverse("@@edit")
        html = view()
        self.assertTrue("form-widgets-ICollection-query" in html)
        # from plone.app.contentlisting.interfaces import IContentListing
        # self.assertTrue(IContentListing.providedBy(view.accessor()))
        # self.assertTrue(getattr(accessor(), 'actual_result_count'))
        # self.assertEqual(accessor().actual_result_count, 0)



from plone.app.contenttypes.utils import replace_link_variables_by_paths
from plone.app.uuid.utils import uuidToObject
from plone.base.interfaces import ITypesSchema
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib.parse import urlparse
from zope.component import getUtility


# links starting with these URL scheme should not be redirected to
NON_REDIRECTABLE_URL_SCHEMES = [
    "mailto:",
    "tel:",
    "callto:",  # nonstandard according to RFC 3966. used for skype.
    "webdav:",
    "caldav:",
]

# links starting with these URL scheme should not be resolved to paths
NON_RESOLVABLE_URL_SCHEMES = NON_REDIRECTABLE_URL_SCHEMES + [
    "file:",
    "ftp:",
]


def normalize_uid_from_path(url=None):
    """
    Args:
        url (string): a path or orl

    Returns:
        tuple: tuple of (uid, fragment) a fragment is an anchor id e.g. #head1
    """
    uid = None
    fragment = None

    if not url:
        return uid, fragment

    # resolve uid
    paths = url.split("/")
    paths_lower = [_item.lower() for _item in paths]

    if "resolveuid" in paths_lower:
        ri = paths_lower.index("resolveuid")
        if ri + 1 != len(paths):
            uid = paths[ri + 1]
            if uid == "":
                uid = None

    if not uid:
        return uid, fragment

    # resolve fragment
    parts = urlparse(uid)

    uid = parts.path

    fragment = f"#{parts.fragment}" if parts.fragment else None

    return uid, fragment


class LinkRedirectView(BrowserView):
    index = ViewPageTemplateFile("templates/link.pt")

    def _url_uses_scheme(self, schemes, url=None):
        url = url or self.context.remoteUrl.strip()
        for scheme in schemes:
            if url.startswith(scheme):
                return True
        return False

    def __call__(self):
        """Redirect to the Link target URL, if and only if:
        - redirect_links property is enabled in
          configuration registry
        - the link is of a redirectable type (no mailto:, etc)
        - AND current user doesn't have permission to edit the Link"""
        context = self.context
        mtool = getToolByName(context, "portal_membership")

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ITypesSchema, prefix="plone")
        redirect_links = settings.redirect_links

        can_edit = mtool.checkPermission("Modify portal content", context)
        redirect_links = redirect_links and not self._url_uses_scheme(
            NON_REDIRECTABLE_URL_SCHEMES
        )

        if redirect_links and not can_edit:
            return self.request.RESPONSE.redirect(self.absolute_target_url())
        return self.index()

    def url(self):
        """Returns the url with link variables replaced."""
        url = replace_link_variables_by_paths(
            self.context, self.context.remoteUrl.strip()
        )
        return url

    def display_link(self):
        """Format the url for display"""

        url = self.url()
        if "resolveuid" in url:
            uid = url.split("/")[-1]
            obj = uuidToObject(uid)
            if obj:
                title = obj.Title()
                meta = "/".join(obj.getPhysicalPath()[2:])
                if not meta.startswith("/"):
                    meta = "/" + meta
                return {
                    "title": title,
                    "meta": meta,
                }

        parsed = urlparse(url)
        if parsed.scheme == "mailto":
            return {
                "title": parsed.path,
                "meta": "",
            }

        return {
            "title": url,
            "meta": "",
        }

    def absolute_target_url(self):
        """Compute the absolute target URL."""
        url = self.url()

        if self._url_uses_scheme(NON_RESOLVABLE_URL_SCHEMES):
            # For non http/https url schemes, there is no path to resolve.
            return url

        if url.startswith("."):
            # we just need to adapt ../relative/links, /absolute/ones work
            # anyway -> this requires relative links to start with ./ or
            # ../
            context_state = self.context.restrictedTraverse("@@plone_context_state")
            url = "/".join([context_state.canonical_object_url(), url])
        else:
            if "resolveuid" in url:
                uid, fragment = normalize_uid_from_path(url)
                obj = uuidToObject(uid)
                if obj is None:
                    # uid can't resolve, return the url
                    return url

                url = obj.absolute_url()
                if fragment is not None:
                    url = f"{url}{fragment}"

            if not url.startswith(("http://", "https://")):
                url = self.request["SERVER_URL"] + url

        return url



from plone.app.contenttypes.browser.utils import Utils
from plone.base.utils import human_readable_size

import re


class FileView(Utils):
    def is_videotype(self):
        ct = self.context.file.contentType
        return "video/" in ct

    def is_audiotype(self):
        ct = self.context.file.contentType
        return "audio/" in ct

    def human_readable_size(self):
        return human_readable_size(self.context.file.getSize())

    def icon(self):
        if "text" in self.context.file.contentType:
            return "contenttype-text"
        if "html" in self.context.file.contentType:
            return "contenttype-text"
        if re.search(r"(archive|zip|compressed)", self.context.file.contentType):
            return "contenttype-archive"
        if "audio" in self.context.file.contentType:
            return "contenttype-audio"
        if "video" in self.context.file.contentType:
            return "contenttype-video"
        if "pdf" in self.context.file.contentType:
            return "contenttype-pdf"
        else:
            return "contenttype-file"



from Acquisition import aq_inner
from plone.memoize.view import memoize
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.MimetypesRegistry.MimeTypeItem import guess_icon_path
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


PREFIX = "++resource++mimetype.icons/"


class IUtils(Interface):
    """ """

    def getMimeTypeIcon(content_file):
        """ """


@implementer(IUtils)
class Utils(BrowserView):
    def _get_mimes(self, content_file):
        # We use 'yield' so iteration can be cut short
        # if the calling code is happy.
        context = aq_inner(self.context)
        mtr = getToolByName(context, "mimetypes_registry")
        if content_file.contentType:
            # this gives a tuple
            yield from mtr.lookup(content_file.contentType)
        if content_file.filename:
            # this gives a single mime type
            yield mtr.lookupExtension(content_file.filename)
        yield from mtr.lookup("application/octet-stream")

    @memoize
    def getMimeTypeIcon(self, content_file):
        # Get possible mime types, and try to find an icon path.
        # Keep the first one, in case there is no good match.
        first = None
        for mime in self._get_mimes(content_file):
            if first is None:
                first = mime
            if hasattr(mime, "icon_path"):
                icon_path = mime.icon_path
                if not icon_path.startswith("++"):
                    icon_path = PREFIX + icon_path
                return icon_path

        if first is None:
            # Probably does not happen in practice.
            return ""
        context = aq_inner(self.context)
        pstate = getMultiAdapter((context, self.request), name="plone_portal_state")
        portal_url = pstate.portal_url()
        return portal_url + "/" + guess_icon_path(first)



from plone.app.contenttypes.interfaces import INewsItem
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.dexterity.interfaces import IDexterityFTI
from plone.testing.zope import Browser
from Products.Five.browser import BrowserView as View
from zope.component import createObject
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager

import unittest


class NewsItemIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer

        alsoProvides(self.request, IPloneAppContenttypesLayer)
        alsoProvides(self.request, IPloneFormLayer)
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="News Item")
        schema = fti.lookupSchema()
        self.assertTrue(schema.getName().endswith("_0_News_1_Item"))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="News Item")
        self.assertNotEqual(None, fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="News Item")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(INewsItem.providedBy(new_object))

    def test_adding(self):
        self.portal.invokeFactory("News Item", "doc1")
        self.assertTrue(INewsItem.providedBy(self.portal["doc1"]))

    def test_view(self):
        self.portal.invokeFactory("News Item", "news_item")
        news_item = self.portal["news_item"]
        news_item.title = "My News Item"
        news_item.description = "This is my news item."
        news_item.text = RichTextValue("Lorem ipsum", "text/plain", "text/html")
        self.request.set("URL", news_item.absolute_url())
        self.request.set("ACTUAL_URL", news_item.absolute_url())
        view = news_item.restrictedTraverse("@@view")

        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)
        self.assertTrue("My News Item" in view())
        self.assertTrue("This is my news item." in view())
        self.assertTrue("Lorem ipsum" in view())

    def test_leadimage_viewlet_does_not_show_up_for_newsitems(self):
        from plone.app.contenttypes.behaviors.leadimage import ILeadImage
        from zope.interface import alsoProvides

        alsoProvides(self.request, ILeadImage)
        view = View(self.portal, self.request)
        manager = queryMultiAdapter(
            (self.portal, self.request, view),
            IViewletManager,
            "plone.abovecontenttitle",
            default=None,
        )
        self.assertTrue(manager)
        manager.update()
        leadimage_viewlet = [
            v for v in manager.viewlets if v.__name__ == "contentleadimage"
        ]
        self.assertEqual(len(leadimage_viewlet), 0)


class NewsItemFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_add_news_item(self):
        self.browser.open(self.portal_url)
        self.browser.getLink("News Item").click()
        self.browser.getControl(name="form.widgets.IDublinCore.title").value = (
            "My news item"
        )
        self.browser.getControl(name="form.widgets.IDublinCore.description").value = (
            "This is my news item."
        )
        self.browser.getControl(name="form.widgets.IShortName.id").value = ""
        self.browser.getControl(name="form.widgets.IRichTextBehavior.text").value = (
            "Lorem Ipsum"
        )
        self.browser.getControl("Save").click()

        self.assertTrue(self.browser.url.endswith("my-news-item/view"))
        self.assertTrue("My news item" in self.browser.contents)
        self.assertTrue("This is my news item" in self.browser.contents)
        self.assertTrue("Lorem Ipsum" in self.browser.contents)

    def test_add_news_item_with_shortname(self):
        self.browser.open(self.portal_url)
        self.browser.getLink("News Item").click()
        self.browser.getControl(name="form.widgets.IDublinCore.title").value = (
            "My news item"
        )
        self.browser.getControl(name="form.widgets.IShortName.id").value = (
            "my-special-news"
        )
        self.browser.getControl("Save").click()

        self.assertTrue(self.browser.url.endswith("my-special-news/view"))



from Acquisition import aq_inner
from lxml import etree
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.base.interfaces.syndication import ISiteSyndicationSettings
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import unittest


query = [
    {
        "i": "Title",
        "o": "plone.app.querystring.operation.string.is",
        "v": "Collection Test Page",
    }
]


class RSSViewTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Folder", "test-folder")
        self.folder = self.portal["test-folder"]
        self.folder.invokeFactory(
            "Document",
            "page1",
            title="Collection Test Page",
        )
        self.folder.invokeFactory("Collection", "collection1")
        self.collection = aq_inner(self.folder["collection1"])
        self.collection.query = query
        self.request.set("URL", self.collection.absolute_url())
        self.request.set("ACTUAL_URL", self.collection.absolute_url())
        # We need to enable syndication globally.
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSyndicationSettings)
        settings.allowed = True
        settings.default_enabled = True

    def assertIsValidRSS(self, rss):
        # XXX: We might want to validate against a DTD or RelaxNG schema here.
        # schema = etree.XMLSchema(schema_root)
        # parser = etree.XMLParser(dtd_validation=True,schema=schema)
        if isinstance(rss, str):
            rss = rss.encode("utf-8")
        parser = etree.XMLParser()
        return etree.fromstring(rss, parser)

    def test_view(self):
        view = self.collection.restrictedTraverse("@@RSS")
        html = view()
        self.assertEqual(view.request.response.status, 200)
        self.assertTrue("Collection Test Page" in html)

    def test_view_is_valid(self):
        view = self.collection.restrictedTraverse("@@RSS")
        result = self.assertIsValidRSS(view())
        self.assertTrue(b"Collection Test Page" in etree.tostring(result))



from datetime import datetime
from datetime import timedelta
from plone.app.contenttypes.browser.folder import FolderView
from plone.app.contenttypes.interfaces import IFolder
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone.app.contenttypes.tests.test_image import dummy_image
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from plone.testing.zope import Browser
from zope.component import createObject
from zope.component import queryUtility

import unittest


class FolderIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="Folder")
        schema = fti.lookupSchema()
        self.assertTrue(schema.getName().endswith("_0_Folder"))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="Folder")
        self.assertNotEqual(None, fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="Folder")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IFolder.providedBy(new_object))

    def test_adding(self):
        self.portal.invokeFactory("Folder", "doc1")
        self.assertTrue(IFolder.providedBy(self.portal["doc1"]))


class FolderViewIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

    def test_result_filtering(self):
        """Test, if portal_state's friendly_types and the result method's
        keyword arguments are included in the query.
        """

        self.portal.invokeFactory("News Item", "newsitem")
        self.portal.invokeFactory("Document", "document")
        view = FolderView(self.portal, self.request)

        # Test, if all results are found.
        view.portal_state.friendly_types = lambda: ["Document", "News Item"]
        res = view.results()
        self.assertEqual(len(res), 2)

        # Test, if friendly_types does filter for types.
        view.portal_state.friendly_types = lambda: ["Document"]
        res = view.results()
        self.assertEqual(len(res), 1)

        # Test, if friendly_types does filter for types.
        view.portal_state.friendly_types = lambda: ["NotExistingType"]
        res = view.results()
        self.assertEqual(len(res), 0)

        # Test, if kwargs filtering is applied.
        view.portal_state.friendly_types = lambda: ["NotExistingType"]
        res = view.results(
            object_provides="plone.app.contenttypes.interfaces.IDocument"
        )
        self.assertEqual(len(res), 1)

    def test_result_batching(self):
        for idx in range(5):
            self.portal.invokeFactory("Document", f"document{idx}")
        request = self.request.clone()
        request.form["b_size"] = 5
        view = FolderView(self.portal, request)

        batch = view.batch()
        self.assertEqual(batch.length, 5)
        self.assertEqual(len([item for item in batch]), 5)
        self.assertFalse(batch.has_next)

        self.portal.invokeFactory("Document", "document5")

        batch = view.batch()
        self.assertEqual(batch.length, 6)
        self.assertEqual(len([item for item in batch]), 6)
        self.assertFalse(batch.has_next)

        self.portal.invokeFactory("Document", "document6")

        batch = view.batch()
        self.assertEqual(batch.length, 5)
        self.assertEqual(len([item for item in batch]), 5)
        self.assertTrue(batch.has_next)
        self.assertEqual(batch.next_item_count, 2)


class FolderFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_add_folder(self):
        self.browser.open(self.portal_url)
        self.browser.getLink(url="http://nohost/plone/++add++Folder").click()
        widget = "form.widgets.IDublinCore.title"
        self.browser.getControl(name=widget).value = "My folder"
        widget = "form.widgets.IShortName.id"
        self.browser.getControl(name=widget).value = ""
        widget = "form.widgets.IDublinCore.description"
        self.browser.getControl(name=widget).value = "This is my folder."
        self.browser.getControl("Save").click()
        self.assertTrue(self.browser.url.endswith("my-folder/view"))
        self.assertTrue("My folder" in self.browser.contents)
        self.assertTrue("This is my folder" in self.browser.contents)

    def test_add_folder_with_shortname(self):
        self.browser.open(self.portal_url)
        self.browser.getLink(url="http://nohost/plone/++add++Folder").click()
        widget = "form.widgets.IDublinCore.title"
        self.browser.getControl(name=widget).value = "My folder"
        widget = "form.widgets.IShortName.id"
        self.browser.getControl(name=widget).value = "my-special-folder"
        self.browser.getControl("Save").click()
        self.assertTrue(self.browser.url.endswith("my-special-folder/view"))


class FolderViewFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal_url = self.portal.absolute_url()
        self.portal.invokeFactory("Folder", id="folder", title="My Folder")
        self.folder = self.portal.folder
        self.folder_url = self.folder.absolute_url()
        self.folder.invokeFactory("Document", id="doc1", title="Document 1")
        import transaction

        transaction.commit()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_folder_view(self):
        self.browser.open(self.folder_url + "/view")
        self.assertIn("My Folder", self.browser.contents)
        self.assertIn("Document 1", self.browser.contents)

    def test_folder_summary_view(self):
        self.browser.open(self.folder_url + "/summary_view")
        self.assertIn("My Folder", self.browser.contents)
        self.assertIn("Document 1", self.browser.contents)

    def test_folder_full_view(self):
        self.browser.open(self.folder_url + "/full_view")
        self.assertIn("My Folder", self.browser.contents)
        self.assertIn("Document 1", self.browser.contents)

    def test_folder_tabular_view(self):
        self.browser.open(self.folder_url + "/tabular_view")
        self.assertIn("My Folder", self.browser.contents)
        self.assertIn("Document 1", self.browser.contents)

    def test_folder_album_view(self):
        self.folder.invokeFactory("Image", id="image1", title="Image 1")
        img1 = self.folder["image1"]
        img1.image = dummy_image()
        import transaction

        transaction.commit()
        self.browser.open(self.folder_url + "/album_view")
        self.assertIn("My Folder", self.browser.contents)
        self.assertIn(
            '<img src="http://nohost/plone/folder/image1/@@images',
            self.browser.contents,
        )

    def test_list_item_wout_title(self):
        """In content listings, if a content object has no title use it's id."""
        self.folder.invokeFactory("Document", id="doc_wout_title")
        import transaction

        transaction.commit()

        # Document should be shown in listing view (and it's siblings)
        self.browser.open(self.folder_url + "/listing_view")
        self.assertIn("doc_wout_title", self.browser.contents)

        # And also in tabular view
        self.browser.open(self.folder_url + "/tabular_view")
        self.assertIn("doc_wout_title", self.browser.contents)

    def test_event_wout_location(self):
        # deactivate "plone.eventlocation" from Event behaviors
        event_fti = self.portal.portal_types.get("Event")
        if not event_fti:
            return
        event_behaviors = list(event_fti.behaviors)
        event_behaviors.remove("plone.eventlocation")
        event_fti.behaviors = tuple(event_behaviors)

        self.folder.invokeFactory(
            "Event",
            id="event_wout_location",
            title="Event without location",
            start=datetime.now() + timedelta(days=1),
        )

        import transaction

        transaction.commit()

        self.browser.open(self.folder_url + "/listing_view")
        self.assertIn("Event without location", self.browser.contents)



from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from plone.testing.zope import Browser
from zope.interface import alsoProvides

import io
import os
import unittest


class LeadImageBehaviorFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = DexterityFTI("leadimagefolder")
        self.portal.portal_types._setObject("leadimagefolder", fti)
        fti.klass = "plone.dexterity.content.Container"
        fti.behaviors = (
            "plone.app.contenttypes.behaviors.leadimage.ILeadImageBehavior",
        )
        self.fti = fti
        alsoProvides(self.portal.REQUEST, IPloneAppContenttypesLayer)
        alsoProvides(self.request, IPloneAppContenttypesLayer)
        from plone.app.contenttypes.behaviors.leadimage import ILeadImage

        alsoProvides(self.request, ILeadImage)
        self.portal.invokeFactory(
            "leadimagefolder", id="leadimagefolder", title="Folder with a lead image"
        )
        import transaction

        transaction.commit()
        # Set up browser
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_lead_image_in_edit_form(self):
        self.browser.open(self.portal_url + "/leadimagefolder/edit")
        self.assertTrue("Lead Image" in self.browser.contents)
        self.assertTrue("Lead Image Caption" in self.browser.contents)

    def test_lead_image_viewlet_shows_up(self):
        self.browser.open(self.portal_url + "/leadimagefolder/edit")
        # Image upload
        file_path = os.path.join(os.path.dirname(__file__), "image.jpg")
        file_ctl = self.browser.getControl(name="form.widgets.ILeadImageBehavior.image")
        with io.FileIO(file_path, "rb") as f:
            file_ctl.add_file(f, "image/png", "image.jpg")
        # Image caption
        self.browser.getControl(
            name="form.widgets.ILeadImageBehavior.image_caption"
        ).value = "My image caption"
        # Submit form
        self.browser.getControl("Save").click()

        self.assertTrue("My image caption" in self.browser.contents)
        self.assertTrue("image.jpg" in self.browser.contents)

        self.assertTrue('<section id="section-leadimage">' in self.browser.contents)

        # But doesn't show up on folder_contents, which is not a default view
        self.browser.open(self.portal_url + "/leadimagefolder/folder_contents")
        self.assertTrue('<section id="section-leadimage">' not in self.browser.contents)



from Acquisition import aq_base
from Acquisition import aq_inner
from plone.app.contenttypes import _
from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from plone.app.contenttypes.interfaces import IFolder
from plone.app.contenttypes.interfaces import IImage
from plone.app.vocabularies.metadatafields import get_field_label
from plone.base.batch import Batch
from plone.base.interfaces import ISecuritySchema
from plone.base.interfaces import ISiteSchema
from plone.base.utils import safe_callable
from plone.event.interfaces import IEvent
from plone.memoize.view import memoize
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.contentprovider.interfaces import IContentProvider

import random


class FolderView(BrowserView):
    text_class = None
    _plone_view = None
    _portal_state = None
    _pas_member = None

    @property
    def plone_view(self):
        if not self._plone_view:
            self._plone_view = getMultiAdapter(
                (self.context, self.request), name="plone"
            )
        return self._plone_view

    @property
    def portal_state(self):
        if not self._portal_state:
            self._portal_state = getMultiAdapter(
                (self.context, self.request), name="plone_portal_state"
            )
        return self._portal_state

    @property
    def pas_member(self):
        if not self._pas_member:
            self._pas_member = getMultiAdapter(
                (self.context, self.request), name="pas_member"
            )
        return self._pas_member

    @property
    def b_size(self):
        b_size = (
            getattr(self.request, "b_size", None)
            or getattr(self.request, "limit_display", None)
            or 20
        )
        return int(b_size)

    @property
    def b_start(self):
        b_start = getattr(self.request, "b_start", None) or 0
        return int(b_start)

    def results(self, **kwargs):
        """Return a content listing based result set with contents of the
        folder.

        :param **kwargs: Any keyword argument, which can be used for catalog
                         queries.
        :type  **kwargs: keyword argument

        :returns: plone.app.contentlisting based result set.
        :rtype: ``plone.app.contentlisting.interfaces.IContentListing`` based
                sequence.
        """
        # Extra filter
        kwargs.update(self.request.get("contentFilter", {}))
        if "object_provides" not in kwargs:  # object_provides is more specific
            kwargs.setdefault("portal_type", self.friendly_types)
        kwargs.setdefault("batch", True)
        kwargs.setdefault("b_size", self.b_size)
        kwargs.setdefault("b_start", self.b_start)
        kwargs.setdefault("orphan", 1)

        listing = aq_inner(self.context).restrictedTraverse("@@folderListing", None)
        if listing is None:
            return []
        results = listing(**kwargs)
        return results

    def batch(self):
        batch = Batch(self.results(), size=self.b_size, start=self.b_start, orphan=1)
        return batch

    def normalizeString(self, text):
        return self.plone_view.normalizeString(text)

    def toLocalizedTime(self, time, long_format=None, time_only=None):
        return self.plone_view.toLocalizedTime(time, long_format, time_only)

    @property
    def friendly_types(self):
        return self.portal_state.friendly_types()

    @property
    def isAnon(self):
        return self.portal_state.anonymous()

    @property
    def navigation_root_url(self):
        return self.portal_state.navigation_root_url()

    @property
    def use_view_action(self):
        registry = getUtility(IRegistry)
        return registry.get("plone.types_use_view_action_in_listings", [])

    @property
    def show_about(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISecuritySchema, prefix="plone")
        show_about = getattr(settings, "allow_anon_views_about", False)
        return show_about or not self.isAnon

    @property
    def text(self):
        textfield = getattr(aq_base(self.context), "text", None)
        text = (
            textfield.output_relative_to(self.context)
            if getattr(textfield, "output_relative_to", None)
            else None
        )
        if text:
            self.text_class = (
                "stx"
                if textfield.mimeType
                in ("text/structured", "text/x-rst", "text/restructured")
                else "plain"
            )
        return text

    @property
    def tabular_fields(self):
        ret = []
        ret.append("Title")
        if self.show_about:
            ret.append("Creator")
        ret.append("Type")
        if self.show_about:
            ret.append("ModificationDate")
        return ret

    def tabular_field_label(self, field):
        """Return the internationalized label (Message object) corresponding
        to the field.
        """
        return get_field_label(field)

    def tabular_fielddata(self, item, fieldname):
        value = getattr(item, fieldname, "")
        if safe_callable(value):
            value = value()
        if fieldname in [
            "CreationDate",
            "ModificationDate",
            "Date",
            "EffectiveDate",
            "ExpirationDate",
            "effective",
            "expires",
            "start",
            "end",
            "created",
            "modified",
            "last_comment_date",
        ]:
            value = self.toLocalizedTime(value, long_format=1)

        return {
            # 'title': _(fieldname, default=fieldname),
            "value": value
        }

    def is_event(self, obj):
        if getattr(obj, "getObject", False):
            obj = obj.getObject()
        return IEvent.providedBy(obj)

    def formatted_date(self, item):
        provider = getMultiAdapter(
            (self.context, self.request, self), IContentProvider, name="formatted_date"
        )
        return provider(item)

    @property
    @memoize
    def album_images(self):
        """Get all images within this folder."""
        provides = [
            IImage.__identifier__,
            ILeadImage.__identifier__,
        ]
        images = self.results(batch=False, object_provides=provides)
        return images

    @property
    @memoize
    def album_folders(self):
        """Get all folders within this folder."""
        images = self.results(batch=False, object_provides=IFolder.__identifier__)
        return images

    @property
    def album_random_image(self):
        """Get random image from this folder."""
        img = None
        images = self.album_images
        if images:
            img = random.choice(images)
        return img

    @property
    def album_number_images(self):
        """Get number of images from this folder."""
        images = self.album_images
        return len(images)

    @property
    def no_items_message(self):
        return _(
            "description_no_items_in_folder",
            default="There are currently no items in this folder.",
        )

    @memoize
    def get_thumb_scale_table(self):
        if getattr(self.context, "suppress_thumbs", False):
            return None
        thsize = getattr(self.context, "thumb_scale_table", None)
        if thsize:
            return thsize
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        if settings.no_thumbs_tables:
            return None
        return settings.thumb_scale_table

    @memoize
    def get_thumb_scale_list(self):
        if getattr(self.context, "suppress_thumbs", False):
            return None
        thsize = getattr(self.context, "thumb_scale_list", None)
        if thsize:
            return thsize
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        if settings.no_thumbs_lists:
            return None
        return settings.thumb_scale_listing

    @memoize
    def get_thumb_scale_summary(self):
        if getattr(self.context, "suppress_thumbs", False):
            return None
        thsize = getattr(self.context, "thumb_scale_summary", None)
        if thsize:
            return thsize
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        if settings.no_thumbs_summary:
            return None
        return settings.thumb_scale_summary

    def show_icons(self):
        return not getattr(self.context, "suppress_icons", False)



from plone.dexterity.interfaces import IDexterityFTI
from zope.component import queryUtility

import logging


logger = logging.getLogger(name="plone.app.contenttypes upgrade")


def update_type_icons(context):
    """Update portal_type icons.

    We want to update two things:

    - the icon_expr property of the portal_type
    - the icon_expr property of the view and edit actions.

    This is for the standard types defined here, plus the Plone Site.

    An earlier version of this upgrade step did this in xml in an upgrade
    profile.  This led to duplicate view and edit actions, because the xml did
    not contain a category for the action.
    A second try with xml still resulted in duplicate view actions,
    and the view and edit actions were invisible.

    Conclusion: if we want to do this in xml, we must specify *all* properties.
    And we do not want this, because these properties may have been changed
    by the user.
    So: we do it in Python.
    """
    types = {
        "Collection": "string:contenttype/collection",
        "Document": "string:contenttype/document",
        "Event": "string:contenttype/event",
        "File": "string:contenttype/file",
        "Folder": "string:contenttype/folder",
        "Image": "string:contenttype/image",
        "Link": "string:contenttype/link",
        "News Item": "string:contenttype/news-item",
        "Plone Site": "string:contenttype/plone",
    }
    view_icon_expr = "string:toolbar-action/view"
    edit_icon_expr = "string:toolbar-action/edit"
    for type_name, icon_expr in types.items():
        fti = queryUtility(IDexterityFTI, name=type_name)
        if not fti:
            continue
        if fti.getProperty("icon_expr") != icon_expr:
            if not fti.hasProperty("icon_expr"):
                fti._setProperty("icon_expr", icon_expr)
            else:
                fti._updateProperty("icon_expr", icon_expr)
            logger.info("Set icon_expr property on FTI %s to %s", type_name, icon_expr)

        # Build up the new actions list.
        new_actions = []
        discarded_actions = []
        found_visible_view = False
        found_visible_edit = False
        changed_icon_expression = False
        for action in fti._actions:
            if not action.category:
                discarded_actions.append(action)
                continue
            if action.category != "object":
                new_actions.append(action)
                continue
            if action.id not in ("view", "edit"):
                new_actions.append(action)
                continue
            # At this point we have id=view/edit and category=object.
            if not action.visible:
                # A previous version of the update may have resulted in
                # an extra, invisible action.
                discarded_actions.append(action)
                continue
            action_icon_expr = action.getIconExpression()
            if action.id == "view":
                found_visible_view = True
                if action_icon_expr != view_icon_expr:
                    # This is what we wanted: change the icon expression.
                    action.setIconExpression(view_icon_expr)
                    changed_icon_expression = True
            elif action.id == "edit":
                found_visible_edit = True
                if action_icon_expr != edit_icon_expr:
                    # This is what we wanted: change the icon expression.
                    action.setIconExpression(edit_icon_expr)
                    changed_icon_expression = True
            new_actions.append(action)
        if not (found_visible_view and found_visible_edit):
            # This is an unexpected situation.
            # If we change something, we might make it worse.
            logger.warning(
                "Did not find both visible view and edit actions for type %s. Not updating type icons.",
                type_name,
            )
            continue
        if not (discarded_actions or changed_icon_expression):
            # no changes needed
            continue
        if discarded_actions:
            logger.info(
                "Removed %d actions without category from FTI %s.",
                len(discarded_actions),
                type_name,
            )
        if changed_icon_expression:
            logger.info(
                "Changed icon expression for view/edit action from FTI %s.",
                type_name,
            )
        fti._actions = tuple(new_actions)



from plone.registry.interfaces import IRegistry
from Products.Five.browser import BrowserView
from zope.component import getUtility
from zope.publisher.interfaces.browser import IBrowserView


class FullViewItem(BrowserView):
    @property
    def default_view(self):
        item_layout = self.context.getLayout()
        default_view = self.context.restrictedTraverse(item_layout)
        return default_view

    @property
    def item_macros(self):
        default_view = self.default_view
        if IBrowserView.providedBy(default_view):
            # IBrowserView
            return default_view.index.macros
        # FSPageTemplate
        return default_view.macros

    @property
    def item_url(self):
        registry = getUtility(IRegistry)
        use_view_action = registry.get("plone.types_use_view_action_in_listings", [])
        url = self.context.absolute_url()
        if self.context.portal_type in use_view_action:
            url = f"{url}/view"
        return url



from pkg_resources import resource_filename


TEST_FOLDER_ID = "robot-test-folder"
PLONE_PATH = "/plone"
COLLECTION_TEST_QUERY = (
    '[{"i": "path", "o": "plone.app.querystring.operation.string.path", "v": "%s/%s"}]'
    % (PLONE_PATH, TEST_FOLDER_ID)
)  # noqa
PATH_TO_TEST_FILES = resource_filename("plone.app.contenttypes.tests", "")



from plone.app.contenttypes import _
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from z3c.form.interfaces import IAddForm
from z3c.form.interfaces import IEditForm
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class IThumbIconHandling(model.Schema):
    model.fieldset(
        "settings",
        label=_("Settings"),
        fields=[
            "thumb_scale_list",
            "thumb_scale_table",
            "thumb_scale_summary",
            "suppress_icons",
            "suppress_thumbs",
        ],
    )

    thumb_scale_list = schema.TextLine(
        title=_("Override thumb scale for list view"),
        description=_(
            "Enter a valid scale name"
            " (see 'Image Handling' control panel) to override"
            " (e.g. icon, tile, thumb, mini, preview, ... )."
            " Leave empty to use default (see 'Site' control panel)."
        ),
        required=False,
        default="",
    )

    thumb_scale_table = schema.TextLine(
        title=_("Override thumb scale for table view"),
        description=_(
            "Enter a valid scale name"
            " (see 'Image Handling' control panel) to override"
            " (e.g. icon, tile, thumb, mini, preview, ... )."
            " Leave empty to use default (see 'Site' control panel)."
        ),
        required=False,
        default="",
    )

    thumb_scale_summary = schema.TextLine(
        title=_("Override thumb scale for summary view"),
        description=_(
            "Enter a valid scale name"
            " (see 'Image Handling' control panel) to override"
            " (e.g. icon, tile, thumb, mini, preview, ... )."
            " Leave empty to use default (see 'Site' control panel)."
        ),
        required=False,
        default="",
    )

    suppress_icons = schema.Bool(
        title=_("Suppress icons in list, table or summary view"),
        description=_(""),
        required=False,
        default=False,
    )

    suppress_thumbs = schema.Bool(
        title=_("Suppress thumbs in list, table or summary view"),
        required=False,
        default=False,
    )

    directives.omitted(
        "thumb_scale_list",
        "thumb_scale_table",
        "thumb_scale_summary",
        "suppress_icons",
        "suppress_thumbs",
    )
    directives.no_omit(
        IEditForm,
        "thumb_scale_list",
        "thumb_scale_table",
        "thumb_scale_summary",
        "suppress_icons",
        "suppress_thumbs",
    )
    directives.no_omit(
        IAddForm,
        "thumb_scale_list",
        "thumb_scale_table",
        "thumb_scale_summary",
        "suppress_icons",
        "suppress_thumbs",
    )



from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class PloneAppContenttypesSetupTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.types = self.portal.portal_types

    def test_old_topic_disabled(self):
        self.assertTrue("Topic" not in self.types.objectIds())

    def test_atcontenttypes_replaced_by_dexterity_types(self):
        self.assertEqual(self.types["Collection"].meta_type, "Dexterity FTI")
        self.assertEqual(self.types["Document"].meta_type, "Dexterity FTI")
        self.assertEqual(self.types["Event"].meta_type, "Dexterity FTI")
        self.assertEqual(self.types["File"].meta_type, "Dexterity FTI")
        self.assertEqual(self.types["Folder"].meta_type, "Dexterity FTI")
        self.assertEqual(self.types["Image"].meta_type, "Dexterity FTI")
        self.assertEqual(self.types["Link"].meta_type, "Dexterity FTI")
        self.assertEqual(self.types["News Item"].meta_type, "Dexterity FTI")

    def test_browserlayer_available(self):
        from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
        from plone.browserlayer import utils

        self.assertTrue(IPloneAppContenttypesLayer in utils.registered_layers())



from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from zope.component import getMultiAdapter

import unittest


# The default fallback is the icon for 'application/octet-stream':
FALLBACK = "++resource++mimetype.icons/application.png"
# Most or all icons should have this as prefix:
PREFIX = "++resource++mimetype.icons/"


class DummyFile:
    """Dummy file object.

    For these tests, we only need a contentType and filename.
    """

    def __init__(self, contentType, filename):
        self.contentType = contentType
        self.filename = filename


class MimeTypeIconIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer["portal"]
        request = self.layer["request"]
        self.view = getMultiAdapter((portal, request), name="contenttype_utils")

    def test_none(self):
        self.assertEqual(self.view.getMimeTypeIcon(DummyFile(None, None)), FALLBACK)

    def test_unknown(self):
        self.assertEqual(
            self.view.getMimeTypeIcon(DummyFile("some/unknown", "unknown.unknown")),
            FALLBACK,
        )

    def test_contenttype_pdf(self):
        self.assertEqual(
            self.view.getMimeTypeIcon(DummyFile("application/pdf", None)),
            PREFIX + "pdf.png",
        )

    def test_filename_pdf(self):
        self.assertEqual(
            self.view.getMimeTypeIcon(DummyFile(None, "plone.pdf")), PREFIX + "pdf.png"
        )



from plone.dexterity.interfaces import IDexterityContent
from plone.dexterity.interfaces import IDexterityFTI
from plone.folder.interfaces import IOrdering
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2Base
from Products.CMFCore.utils import getToolByName
from zExceptions import NotFound
from zope.component import queryUtility
from zope.interface import alsoProvides

import importlib
import logging


logger = logging.getLogger(__name__)


DEFAULT_TYPES = [
    "Collection",
    "Document",
    "Event",
    "File",
    "Folder",
    "Image",
    "Link",
    "News Item",
]


def replace_link_variables_by_paths(context, url):
    """Take an `url` and replace the variables "${navigation_root_url}" and
    "${portal_url}" by the corresponding paths. `context` is the acquisition
    context.
    """
    if not url:
        return url

    portal_state = context.restrictedTraverse("@@plone_portal_state")

    if "${navigation_root_url}" in url:
        url = _replace_variable_by_path(
            url, "${navigation_root_url}", portal_state.navigation_root()
        )

    if "${portal_url}" in url:
        url = _replace_variable_by_path(url, "${portal_url}", portal_state.portal())

    return url


def _replace_variable_by_path(url, variable, obj):
    path = "/".join(obj.getPhysicalPath())
    return url.replace(variable, path)


def get_old_class_name_string(obj):
    """Returns the current class name string."""
    return f"{obj.__module__}.{obj.__class__.__name__}"


def get_portal_type_name_string(obj):
    """Returns the klass-attribute of the fti."""
    fti = queryUtility(IDexterityFTI, name=obj.portal_type)
    if not fti:
        return False
    return fti.klass


def migrate_base_class_to_new_class(
    obj,
    indexes=None,
    old_class_name="",
    new_class_name="",
    migrate_to_folderish=False,
):
    if indexes is None:
        indexes = ["is_folderish", "object_provides"]
    if not old_class_name:
        old_class_name = get_old_class_name_string(obj)
    if not new_class_name:
        new_class_name = get_portal_type_name_string(obj)
        if not new_class_name:
            logger.warning(f"The type {obj.portal_type} has no fti!")
            return False

    was_item = not isinstance(obj, BTreeFolder2Base)
    if old_class_name != new_class_name:
        obj_id = obj.getId()
        module_name, class_name = new_class_name.rsplit(".", 1)
        module = importlib.import_module(module_name)
        new_class = getattr(module, class_name)

        # update obj class
        parent = obj.__parent__
        parent._delOb(obj_id)
        obj.__class__ = new_class
        parent._setOb(obj_id, obj)

    is_container = isinstance(obj, BTreeFolder2Base)

    if was_item and is_container or migrate_to_folderish and is_container:
        alsoProvides(obj, IOrdering)
        #  If Itemish becomes Folderish we have to update obj _tree
        BTreeFolder2Base._initBTrees(obj)

    # reindex
    obj.reindexObject(indexes)

    return True


def list_of_objects_with_changed_base_class(context):
    catalog = getToolByName(context, "portal_catalog")
    for brain in catalog(object_provides=IDexterityContent.__identifier__):
        try:
            obj = brain.getObject()
        except (KeyError, NotFound):
            logger.warn(f"Object {brain.getPath()} not found")
            continue
        if get_portal_type_name_string(obj) != get_old_class_name_string(obj):
            yield obj


def changed_base_classes(context):
    """Returns dict of current and new class names ."""
    results = {}
    for obj in list_of_objects_with_changed_base_class(context):
        current_class_name = get_old_class_name_string(obj)
        new_class_name = get_portal_type_name_string(obj)
        if current_class_name not in results:
            results[current_class_name] = {
                "old": current_class_name,
                "new": new_class_name,
                "amount": 1,
            }
        else:
            results[current_class_name]["amount"] += 1
    return results



from datetime import datetime
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.dexterity.interfaces import IDexterityFTI
from plone.event.interfaces import IEvent
from plone.testing.zope import Browser
from zope.component import createObject
from zope.component import queryUtility
from zope.interface import alsoProvides

import unittest


class EventIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        self.request["LANGUAGE"] = "en"
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="Event")
        schema = fti.lookupSchema()
        self.assertTrue(schema.getName().endswith("_0_Event"))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="Event")
        self.assertNotEqual(None, fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="Event")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IEvent.providedBy(new_object))

    def test_adding(self):
        self.portal.invokeFactory("Event", "doc1")
        self.assertTrue(IEvent.providedBy(self.portal["doc1"]))

    def test_view(self):
        self.portal.invokeFactory("Event", "event")
        event = self.portal["event"]
        event.title = "My Event"
        event.description = "This is my event."
        event.start = datetime(2013, 1, 1, 10, 0)
        event.end = datetime(2013, 1, 1, 12, 0)

        self.request.set("URL", event.absolute_url())
        self.request.set("ACTUAL_URL", event.absolute_url())
        alsoProvides(self.request, IPloneFormLayer)
        view = event.restrictedTraverse("@@view")

        # TODO: start/end are not set??
        #
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)
        self.assertTrue("My Event" in view())
        self.assertTrue("This is my event." in view())


class EventFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_add_event(self):
        self.browser.open(self.portal_url)
        self.browser.getLink("Event").click()
        self.browser.getControl(name="form.widgets.IDublinCore.title").value = (
            "My event"
        )
        self.browser.getControl(name="form.widgets.IDublinCore.description").value = (
            "This is my event."
        )
        self.browser.getControl(name="form.widgets.IRichTextBehavior.text").value = (
            "Lorem Ipsum"
        )
        self.browser.getControl(name="form.widgets.IEventBasic.start").value = (
            "2013-01-01"
        )
        self.browser.getControl(name="form.widgets.IEventBasic.end").value = (
            "2013-01-12"
        )
        self.browser.getControl(name="form.widgets.IShortName.id").value = (
            "my-special-event"
        )
        self.browser.getControl("Save").click()

        self.assertTrue(self.browser.url.endswith("my-special-event/view"))
        self.assertIn("My event", self.browser.contents)
        self.assertIn("This is my event", self.browser.contents)
        self.assertIn("Lorem Ipsum", self.browser.contents)
        self.assertIn("2013-01-01", self.browser.contents)
        self.assertIn("2013-01-12", self.browser.contents)



from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from plone.testing.zope import Browser
from zope.interface import alsoProvides

import json
import unittest


query = [
    {
        "i": "Title",
        "o": "plone.app.querystring.operation.string.contains",
        "v": "Collection Test Page",
    }
]


class CollectionBehaviorFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        fti = DexterityFTI("collectioncontainer")
        self.portal.portal_types._setObject("collectioncontainer", fti)
        fti.klass = "plone.dexterity.content.Container"
        fti.behaviors = ("plone.app.contenttypes.behaviors.collection.ICollection",)
        self.fti = fti
        alsoProvides(self.portal.REQUEST, IPloneAppContenttypesLayer)
        alsoProvides(self.request, IPloneAppContenttypesLayer)
        from plone.app.contenttypes.behaviors.collection import ICollection

        alsoProvides(self.request, ICollection)
        self.portal.invokeFactory(
            "collectioncontainer",
            id="collectioncontainer",
            title="Container with a collection",
            customViewFields=["Title", "portal_type"],
            query=query,
        )
        self.portal.invokeFactory(
            "Document",
            id="doc",
            title="Collection Test Page",
        )

    def _get_browser(self):
        # Need to commit transaction, otherwise the browser does not
        # see anything.
        import transaction

        transaction.commit()
        # Set up browser
        app = self.layer["app"]
        browser = Browser(app)
        browser.handleErrors = False
        browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )
        return browser

    def test_collection_default_dexterity_view(self):
        # Test the default dexterity view
        browser = self._get_browser()
        browser.open(self.portal_url + "/collectioncontainer/view")
        self.assertTrue('"v": "Collection Test Page"' in browser.contents)

    def test_collection_tabular_view(self):
        browser = self._get_browser()
        browser.open(self.portal_url + "/collectioncontainer/tabular_view")
        # search from here:
        start = browser.contents.find("content-core")
        # The test string should be within the search results.
        self.assertTrue(
            "Collection Test Page" in browser.contents[start : start + 2000]
        )

    def test_collection_in_edit_form(self):
        browser = self._get_browser()
        browser.open(self.portal_url + "/collectioncontainer/edit")
        control = browser.getControl(name="form.widgets.ICollection.query")
        self.assertTrue(json.loads(control.value)[0]["v"], "Collection Test Page")
        # The customViewFields field is a 'double' control, with a
        # 'from' and 'to' list.
        from_control = browser.getControl(
            name="form.widgets.ICollection.customViewFields.from"
        )
        self.assertEqual(from_control.value, [])
        self.assertTrue("Title" not in from_control.options)
        self.assertTrue("portal_type" not in from_control.options)
        self.assertTrue("Description" in from_control.options)



from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from plone.app.contenttypes.tests.robot.variables import TEST_FOLDER_ID
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.testing import zope
from zope.interface import alsoProvides


def set_browserlayer(request):
    """Set the BrowserLayer for the request.

    We have to set the browserlayer manually, since importing the profile alone
    doesn't do it in tests.
    """
    alsoProvides(request, IPloneAppContenttypesLayer)


class PloneAppContenttypes(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.contenttypes

        self.loadZCML(package=plone.app.contenttypes)

    def setUpPloneSite(self, portal):
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")


class PloneAppContenttypesRobot(PloneAppContenttypes):
    """Same as the default but with a added folder 'robot-test-folder'."""

    defaultBases = (REMOTE_LIBRARY_BUNDLE_FIXTURE,)

    def setUpPloneSite(self, portal):
        portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ["Manager"], []
        )
        login(portal, SITE_OWNER_NAME)
        super().setUpPloneSite(portal)
        portal.invokeFactory("Folder", id=TEST_FOLDER_ID, title="Test Folder")

    def tearDownPloneSite(self, portal):
        login(portal, "admin")
        portal.manage_delObjects([TEST_FOLDER_ID])
        super().tearDownPloneSite(portal)


PLONE_APP_CONTENTTYPES_FIXTURE = PloneAppContenttypes()
PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,), name="PloneAppContenttypes:Integration"
)
PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,), name="PloneAppContenttypes:Functional"
)
PLONE_APP_CONTENTTYPES_ROBOT_FIXTURE = PloneAppContenttypesRobot()
PLONE_APP_CONTENTTYPES_ROBOT_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTTYPES_ROBOT_FIXTURE, zope.WSGI_SERVER_FIXTURE),
    name="PloneAppContenttypes:Robot",
)



from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from plone.testing.zope import Browser
from zope.interface import alsoProvides

import unittest


class TableOfContentsBehaviorFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = DexterityFTI("tocdocument")
        self.portal.portal_types._setObject("tocdocument", fti)
        fti.klass = "plone.dexterity.content.Item"
        fti.behaviors = ("plone.tableofcontents", "plone.richtext")
        self.fti = fti
        alsoProvides(self.request, IPloneAppContenttypesLayer)
        self.portal.invokeFactory(
            "tocdocument",
            id="tocdoc",
            title="Document with a table of contents",
        )
        import transaction

        transaction.commit()
        # Set up browser
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_toc_in_edit_form(self):
        self.browser.open(self.portal_url + "/tocdoc/edit")
        self.assertTrue("Table of contents" in self.browser.contents)

    def test_toc_viewlet_shows_up(self):
        self.browser.open(self.portal_url + "/tocdoc/edit")
        toc_ctl = self.browser.getControl(
            name="form.widgets.ITableOfContents.table_of_contents:list"
        )
        toc_ctl.value = [
            "selected",
        ]
        # Submit form
        self.browser.getControl("Save").click()
        self.assertTrue('id="document-toc"' in self.browser.contents)



from io import BytesIO
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME

import base64
import re
import sys
import transaction
import unittest


# List various possibly read methods.
# Not all classes have all of them.
# Some may be inherited.
READ_METHODS = (
    "content_type",
    "Format",
    "get_size",
    "getFoldersAndImages",
    "getQuery",
    "getRawQuery",
    "index_html",
    "listMetaDataFields",
    "manage_DAVget",
    "manage_FTPget",
    "queryCatalog",
    "results",
    "selectedViewFields",
)
WRITE_METHODS = ("setQuery", "setSort_on", "setSort_reversed", "PUT")


class ResponseWrapper:
    """Decorates a response object with additional introspective methods."""

    _bodyre = re.compile("\r\n\r\n(.*)", re.MULTILINE | re.DOTALL)

    def __init__(self, response, outstream, path):
        self._response = response
        self._outstream = outstream
        self._path = path

    def __getattr__(self, name):
        return getattr(self._response, name)

    def getOutput(self):
        """Returns the complete output, headers and all."""
        return self._outstream.getvalue()

    def getBody(self):
        """Returns the page body, i.e. the output par headers."""
        body = self._bodyre.search(self.getOutput())
        if body is not None:
            body = body.group(1)
        return body

    def getPath(self):
        """Returns the path used by the request."""
        return self._path

    def getHeader(self, name):
        """Returns the value of a response header."""
        return self.headers.get(name.lower())

    def getCookie(self, name):
        """Returns a response cookie."""
        return self.cookies.get(name)


class TestSecurity(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        # self.request['ACTUAL_URL'] = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def publish(
        self,
        path,
        basic=None,
        env=None,
        extra=None,
        request_method="GET",
        stdin=None,
        handle_errors=True,
    ):
        """
        Mostly pulled from Testing.functional
        """
        # Note: the next import fail in Python 3, because it needs ZServer.
        from ZPublisher.Publish import publish_module
        from ZPublisher.Request import Request
        from ZPublisher.Response import Response

        transaction.commit()

        if env is None:
            env = {}

        env["SERVER_NAME"] = self.request["SERVER_NAME"]
        env["SERVER_PORT"] = self.request["SERVER_PORT"]
        env["REQUEST_METHOD"] = request_method

        p = path.split("?")
        if len(p) == 1:
            env["PATH_INFO"] = p[0]
        elif len(p) == 2:
            [env["PATH_INFO"], env["QUERY_STRING"]] = p
        else:
            raise TypeError("")

        if basic:
            env["HTTP_AUTHORIZATION"] = "Basic %s" % base64.encodestring(basic)

        if stdin is None:
            stdin = BytesIO()

        outstream = BytesIO()
        response = Response(stdout=outstream, stderr=sys.stderr)
        request = Request(stdin, env, response)

        publish_module(
            "Zope2", debug=not handle_errors, request=request, response=response
        )

        return ResponseWrapper(response, outstream, path)

    def test_put_gives_401(self):
        try:
            # pkg_resources.get_distribution("ZServer") is not good enough,
            # because ZServer may be included in the Zope2 package.
            import ZServer  # noqa
        except ImportError:
            print("Ignoring PUT request method tests, as we miss the ZServer.")
            return

        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Collection", id="collection")
        self.portal.invokeFactory("Document", id="page")
        self.portal.invokeFactory("File", id="file")
        self.portal.invokeFactory("Folder", id="folder")
        self.portal.invokeFactory("Image", id="image")
        collection = self.portal.collection
        fi = self.portal.file
        folder = self.portal.folder
        image = self.portal.image
        page = self.portal.page
        logout()

        # from zope.publisher.browser import TestRequest

        # import pdb; pdb.set_trace()
        # request = TestRequest(request_method="PUT")
        # collection.REQUEST = request
        # x = collection()

        path = "/" + collection.absolute_url(relative=True)
        response = self.publish(path=path, env={}, request_method="PUT")
        self.assertEqual(response.getStatus(), 401)

        path = "/" + fi.absolute_url(relative=True)
        response = self.publish(path=path, env={}, request_method="PUT")
        self.assertEqual(response.getStatus(), 401)

        path = "/" + folder.absolute_url(relative=True)
        response = self.publish(path=path, env={}, request_method="PUT")
        self.assertEqual(response.getStatus(), 401)

        path = "/" + image.absolute_url(relative=True)
        response = self.publish(path=path, env={}, request_method="PUT")
        self.assertEqual(response.getStatus(), 401)

        path = "/" + page.absolute_url(relative=True)
        response = self.publish(path=path, env={}, request_method="PUT")
        self.assertEqual(response.getStatus(), 401)

    def DISABLED_test_listDAVobjects_gives_401(self):
        # This actually gives 302, both with and without the patch.  It is
        # protected with AccessControl.Permissions.webdav_access.
        login(self.portal, TEST_USER_NAME)
        # only defined for folderish items
        self.portal.invokeFactory("Folder", id="folder")
        folder = self.portal.folder
        logout()

        folder_path = "/" + folder.absolute_url(relative=True)
        path = folder_path + "/listDAVObjects"
        response = self.publish(path=path, env={}, request_method="GET")
        self.assertEqual(response.getStatus(), 401)

    def get_permission_mapping(self, klass):
        permissions = klass.__ac_permissions__
        mapping = {}
        for permission in permissions:
            # permission can have two or three items:
            # ('WebDAV access',
            #  ('PROPFIND', 'listDAVObjects', 'manage_DAVget'),
            #  ('Manager', 'Authenticated'))
            perm, methods = list(permission)[:2]
            for method in methods:
                mapping[method] = perm
        return mapping

    def _test_class_protected(self, klass):
        mapping = self.get_permission_mapping(klass)
        for method in READ_METHODS:
            if method in klass.__dict__.keys():
                self.assertEqual(
                    mapping.get(method),
                    "View",
                    f"Method {method} missing view protection",
                )
        for method in WRITE_METHODS:
            if method in klass.__dict__.keys():
                self.assertEqual(
                    mapping.get(method),
                    "Modify portal content",
                    f"Method {method} missing edit protection",
                )

    def testCollection_protected(self):
        try:
            from plone.app.contenttypes.content import Collection
        except ImportError:
            return
        self._test_class_protected(Collection)

    def testDocument_protected(self):
        try:
            from plone.app.contenttypes.content import Document
        except ImportError:
            return
        self._test_class_protected(Document)

    def testFile_protected(self):
        try:
            from plone.app.contenttypes.content import File
        except ImportError:
            return
        self._test_class_protected(File)

    def testImage_protected(self):
        try:
            from plone.app.contenttypes.content import Image
        except ImportError:
            return
        self._test_class_protected(Image)



from Acquisition import aq_base
from plone.app.contenttypes import _
from plone.app.dexterity.textindexer import searchable
from plone.app.textfield import RichText as RichTextField
from plone.app.z3cform.widget import RichTextFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.autoform.view import WidgetsView
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class IRichText(Interface):
    pass


@provider(IFormFieldProvider)
class IRichTextBehavior(model.Schema):
    text = RichTextField(
        title=_("Text"),
        description="",
        required=False,
    )
    form.widget("text", RichTextFieldWidget)
    model.primary("text")
    searchable("text")


@implementer(IRichTextBehavior)
@adapter(IDexterityContent)
class RichText:
    def __init__(self, context):
        self.context = context

    @property
    def text(self):
        return getattr(aq_base(self.context), "text", "")

    @text.setter
    def text(self, value):
        self.context.text = value


class WidgetView(WidgetsView):
    schema = IRichTextBehavior



from plone.app.contenttypes.utils import changed_base_classes
from plone.app.contenttypes.utils import get_old_class_name_string
from plone.app.contenttypes.utils import migrate_base_class_to_new_class
from plone.base import PloneMessageFactory as _
from plone.z3cform.layout import wrap_form
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from zExceptions import NotFound
from zope import schema
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary

import logging


logger = logging.getLogger(__name__)


@implementer(IVocabularyFactory)
class ChangedBaseClasses:
    def __call__(self, context):
        """Return a vocabulary with all changed base classes."""
        terms = []
        for class_name, data in changed_base_classes(context).items():
            title = "{} (➡ {}) - ({} items)".format(
                data["old"], data["new"], data["amount"]
            )
            term = SimpleVocabulary.createTerm(class_name, class_name, title)
            terms.append(term)
        return SimpleVocabulary(terms)


class IBaseClassMigratorForm(Interface):
    changed_base_classes = schema.List(
        title="Changed base classes (old class, new class and number of items)",
        description="Select changed base classes you want to migrate. "
        "If the new types are folderish that change is also applied.",
        value_type=schema.Choice(
            vocabulary="plone.app.contenttypes.migration.changed_base_classes",
        ),
        default=[],
        required=True,
    )


class BaseClassMigratorForm(form.Form):
    label = _(
        "heading_class_migrator",
        default="Update base-classes for content with changed classes",
    )

    fields = field.Fields(IBaseClassMigratorForm)
    fields["changed_base_classes"].widgetFactory = CheckBoxFieldWidget
    ignoreContext = True
    enableCSRFProtection = True

    def updateWidgets(self):
        super().updateWidgets()
        changed_base_classes = self.widgets["changed_base_classes"]
        if not changed_base_classes.terms.terms.by_value:
            IStatusMessage(self.request).addStatusMessage(
                "No types with changed classes to migrate!", type="warning"
            )
            return

    @button.buttonAndHandler("Update", name="update")
    def handle_migrate(self, action):
        data, errors = self.extractData()

        if errors:
            return

        changed_base_classes = data.get("changed_base_classes", [])
        messages = IStatusMessage(self.request)
        if not changed_base_classes:
            messages.addStatusMessage("No types were selected", type="warning")
            return

        catalog = getToolByName(self.context, "portal_catalog")
        migrated = []
        not_migrated = []
        for brain in catalog():
            try:
                obj = brain.getObject()
            except (KeyError, NotFound):
                continue
            old_class_name = get_old_class_name_string(obj)
            if old_class_name in changed_base_classes:
                if migrate_base_class_to_new_class(obj, migrate_to_folderish=True):
                    migrated.append(obj)
                else:
                    not_migrated.append(obj)

        info_message_template = "{0} objects were migrated."
        warn_message_template = "{0} objects were not migrated."
        if migrated:
            msg = info_message_template.format(len(migrated))
            messages.addStatusMessage(msg, type="info")
        if not_migrated:
            msg = warn_message_template.format(len(not_migrated))
            messages.addStatusMessage(msg, type="warning")
        self.request.response.redirect(self.request["ACTUAL_URL"])


BaseClassMigrator = wrap_form(
    BaseClassMigratorForm,
)



from plone.app.contenttypes.behaviors.richtext import IRichText
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from plone.testing.zope import Browser
from Products.CMFCore.utils import getToolByName

import unittest


class RichTextBase:
    # subclass here
    _behaviors = None
    _portal_type = None

    def _setupFTI(self):
        fti = DexterityFTI(self._portal_type)
        self.portal.portal_types._setObject(self._portal_type, fti)
        fti.klass = "plone.dexterity.content.Item"
        fti.behaviors = self._behaviors


class RichTextBehaviorFunctionalTest(RichTextBase, unittest.TestCase):
    """basic use cases and tests for richtext behavior"""

    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    _behaviors = ("plone.app.contenttypes.behaviors.richtext.IRichTextBehavior",)
    _portal_type = "SomeDocument"

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.wf = getToolByName(self.portal, "portal_workflow")
        self.portal.acl_users._doAddUser("user_std", "secret", ["Member"], [])
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self._setupFTI()
        self.portal.invokeFactory(self._portal_type, "doc1")
        setRoles(self.portal, TEST_USER_ID, ["Member"])
        import transaction

        transaction.commit()
        # Set up browser
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_richtext_in_edit_form(self):
        self.browser.open(self.portal_url + "/doc1/edit")
        self.assertTrue("tinymce" in self.browser.contents)

    def test_richtext_behavior(self):
        IRichText.providedBy(self.portal.doc1)



from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.i18nmessageid import MessageFactory
from zope.interface import provider


_ = MessageFactory("plone")


@provider(IFormFieldProvider)
class ITableOfContents(model.Schema):
    model.fieldset("settings", label=_("Settings"), fields=["table_of_contents"])

    table_of_contents = schema.Bool(
        title=_("help_enable_table_of_contents", default="Table of contents"),
        description=_(
            "help_enable_table_of_contents_description",
            default="If selected, this will show a table of contents"
            " at the top of the page.",
        ),
        required=False,
    )



from zope.interface import Interface


class IPloneAppContenttypesLayer(Interface):
    """Marker interface that defines a ZTK browser layer. We can reference
    this in the 'layer' attribute of ZCML <browser:* /> directives to ensure
    the relevant registration only takes effect when this theme is installed.

    The browser layer is installed via the browserlayer.xml GenericSetup
    import step.
    """


class ICollection(Interface):
    """Explicit marker interface for Collection"""


class IDocument(Interface):
    """Explicit marker interface for Document"""


class IFile(Interface):
    """Explicit marker interface for File"""


class IFolder(Interface):
    """Explicit marker interface for Folder"""


class IImage(Interface):
    """Explicit marker interface for Image"""


class ILink(Interface):
    """Explicit marker interface for Link"""


class INewsItem(Interface):
    """Explicit marker interface for News Item"""


class IEvent(Interface):
    """Explicit marker interface for Event"""



from plone.app.contenttypes.interfaces import IImage


def set_title_description(obj, event):
    """Sets title to filename if no title
    was provided.
    Also sets an empty unicode as description if
    no description was provided.
    """
    title = obj.title
    if not title:
        if IImage.providedBy(obj):
            datafield = obj.image
        else:
            datafield = obj.file
        if datafield:
            filename = datafield.filename
            obj.title = filename

    description = obj.description
    if not description:
        obj.description = ""



from plone.app.contenttypes import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile import field as namedfile
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class ILeadImage(Interface):
    pass


@provider(IFormFieldProvider)
class ILeadImageBehavior(model.Schema):
    image = namedfile.NamedBlobImage(
        title=_("label_leadimage", default="Lead Image"),
        description="",
        required=False,
    )

    image_caption = schema.TextLine(
        title=_("label_leadimage_caption", default="Lead Image Caption"),
        description="",
        required=False,
    )


@implementer(ILeadImageBehavior)
@adapter(IDexterityContent)
class LeadImage:
    def __init__(self, context):
        self.context = context

    @property
    def image(self):
        return self.context.image

    @image.setter
    def image(self, value):
        self.context.image = value

    @property
    def image_caption(self):
        return self.context.image_caption

    @image_caption.setter
    def image_caption(self, value):
        self.context.image_caption = value



from setuptools import find_packages
from setuptools import setup

import os


version = "3.0.6"


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


long_description = read("README.rst") + "\n\n" + read("CHANGES.rst")


setup(
    name="plone.app.contenttypes",
    version=version,
    description="Default content types for Plone based on Dexterity",
    long_description=long_description,
    # Get more strings from https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "Framework :: Zope :: 5",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords="plone content types dexterity",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/plone.app.contenttypes",
    license="GPL",
    packages=find_packages(),
    namespace_packages=["plone", "plone.app"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "plone.app.contentmenu",
        "plone.app.dexterity >= 2.0.7",  # has a fix for INameFromFilename
        "plone.app.linkintegrity",
        "plone.app.querystring >= 1.2.2",  # custom_query support
        "plone.dexterity >= 2.2.1",  # behaviors can provide primaryfields
        "plone.app.relationfield",
        "plone.namedfile >= 4.2.0",
        "plone.app.versioningbehavior",
        "plone.app.vocabularies > 4.1.2",
        "plone.app.lockingbehavior",
        "plone.behavior >= 1.3.0",
        "plone.app.z3cform>=1.1.0",
        "plone.base",
        "Products.BTreeFolder2",
        "Products.GenericSetup",
        "Products.MimetypesRegistry",
        "Products.PortalTransforms",
        "Products.statusmessages",
        "plone.app.layout",
        "plone.app.textfield",
        "plone.app.uuid",
        "plone.autoform",
        "plone.event",
        "plone.folder",
        "plone.i18n",
        "plone.indexer",
        "plone.memoize",
        "plone.portlets",
        "plone.registry",
        "plone.rfc822",
        "plone.supermodel",
        "plone.z3cform",
        "z3c.form",
        "Zope",
    ],
    extras_require={
        "test": [
            "lxml",
            "plone.app.robotframework [debug] > 0.9.8",  # create image and file content for Image, File and News Item if not given.  # noqa
            "plone.app.testing [robot] >= 4.2.4",  # we need ROBOT_TEST_LEVEL
            "plone.testing",
            "plone.browserlayer",
            "plone.uuid",
            "robotsuite",
        ],
    },
)



from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from plone.app.layout.viewlets import ViewletBase


class LeadImageViewlet(ViewletBase):
    """A simple viewlet which renders leadimage"""

    def update(self):
        self.context = ILeadImage(self.context)
        self.available = True if self.context.image else False



from AccessControl.Permission import addPermission
from AccessControl.SecurityInfo import ModuleSecurityInfo
from plone.app.contenttypes.utils import DEFAULT_TYPES


security = ModuleSecurityInfo("plone.app.contenttypes")

TYPE_ROLES = ("Manager", "Site Administrator", "Owner", "Contributor")

perms = []

for typename in DEFAULT_TYPES:
    permid = "Add" + typename
    permname = "plone.app.contenttypes: Add " + typename
    security.declarePublic(permid)
    addPermission(permname, default_roles=TYPE_ROLES)

AddCollection = "plone.app.contenttypes: Add Collection"
AddDocument = "plone.app.contenttypes: Add Document"
AddEvent = "plone.app.contenttypes: Add Event"
AddFile = "plone.app.contenttypes: Add File"
AddFolder = "plone.app.contenttypes: Add Folder"
AddImage = "plone.app.contenttypes: Add Image"
AddLink = "plone.app.contenttypes: Add Link"
AddNewsItem = "plone.app.contenttypes: Add News Item"



from AccessControl.SecurityInfo import ClassSecurityInfo
from email.feedparser import headerRE
from plone.app.contenttypes.interfaces import ICollection
from plone.app.contenttypes.interfaces import IDocument
from plone.app.contenttypes.interfaces import IEvent
from plone.app.contenttypes.interfaces import IFile
from plone.app.contenttypes.interfaces import IFolder
from plone.app.contenttypes.interfaces import IImage
from plone.app.contenttypes.interfaces import ILink
from plone.app.contenttypes.interfaces import INewsItem
from plone.base.utils import safe_text
from plone.dexterity.content import Container
from plone.dexterity.content import Item
from plone.namedfile.file import NamedBlobFile
from plone.namedfile.file import NamedBlobImage
from Products.CMFCore import permissions
from zope.deprecation import deprecation
from zope.interface import implementer
from zope.lifecycleevent import modified


@implementer(ICollection)
class Collection(Item):
    """Convenience subclass for ``Collection`` portal type"""

    security = ClassSecurityInfo()

    # BBB

    @security.protected(permissions.View)
    def listMetaDataFields(self, exclude=True):
        """Return a list of all metadata fields from portal_catalog.

        This is no longer used.  We use a vocabulary instead.
        """
        return []

    @security.protected(permissions.View)
    def selectedViewFields(self):
        """Returns a list of all metadata fields from the catalog that were
        selected.
        """
        from plone.app.contenttypes.behaviors.collection import (
            ICollection as ICollection_behavior,
        )

        return ICollection_behavior(self).selectedViewFields()

    @security.protected(permissions.ModifyPortalContent)
    def setQuery(self, query):
        self.query = query

    @security.protected(permissions.View)
    def getQuery(self):
        """Return the query as a list of dict; note that this method
        returns a list of CatalogContentListingObject in
        Products.ATContentTypes.
        """
        return self.query

    @deprecation.deprecate("getRawQuery() is deprecated; use getQuery().")
    @security.protected(permissions.View)
    def getRawQuery(self):
        return self.getQuery()

    @security.protected(permissions.ModifyPortalContent)
    def setSort_on(self, sort_on):
        self.sort_on = sort_on

    @security.protected(permissions.ModifyPortalContent)
    def setSort_reversed(self, sort_reversed):
        self.sort_reversed = sort_reversed

    @security.protected(permissions.View)
    def queryCatalog(self, batch=True, b_start=0, b_size=30, sort_on=None):
        from plone.app.contenttypes.behaviors.collection import (
            ICollection as ICollection_behavior,
        )

        return ICollection_behavior(self).results(
            batch, b_start, b_size, sort_on=sort_on
        )

    @security.protected(permissions.View)
    def results(self, **kwargs):
        from plone.app.contenttypes.behaviors.collection import (
            ICollection as ICollection_behavior,
        )

        return ICollection_behavior(self).results(**kwargs)


@implementer(IDocument)
class Document(Item):
    """Convenience subclass for ``Document`` portal type"""

    security = ClassSecurityInfo()

    @security.protected(permissions.View)
    def Format(self):
        """Provide a proper accessor for the format attribute
        See https://github.com/plone/Products.CMFPlone/issues/2540
        """
        return self.format


@implementer(IFile)
class File(Item):
    """Convenience subclass for ``File`` portal type"""

    security = ClassSecurityInfo()

    @security.protected(permissions.ModifyPortalContent)
    def PUT(self, REQUEST=None, RESPONSE=None):
        """DAV method to replace the file field with a new resource."""
        request = REQUEST if REQUEST is not None else self.REQUEST
        response = RESPONSE if RESPONSE is not None else request.response

        infile = request.get("BODYFILE", None)
        first_line = infile.readline()
        infile.seek(0)
        if not headerRE.match(safe_text(first_line)):
            self.dav__init(request, response)
            self.dav__simpleifhandler(request, response, refresh=1)

            filename = request["PATH_INFO"].split("/")[-1]
            self.file = NamedBlobFile(data=infile.read(), filename=str(filename))

            modified(self)
            return response
        else:
            return super().PUT(REQUEST=request, RESPONSE=response)

    @security.protected(permissions.View)
    def get_size(self):
        return getattr(self.file, "size", 0)

    @security.protected(permissions.View)
    def content_type(self):
        return getattr(self.file, "contentType", None)


@implementer(IFolder)
class Folder(Container):
    """Convenience subclass for ``Folder`` portal type"""


@implementer(IImage)
class Image(Item):
    """Convenience subclass for ``Image`` portal type"""

    security = ClassSecurityInfo()

    @security.protected(permissions.ModifyPortalContent)
    def PUT(self, REQUEST=None, RESPONSE=None):
        """DAV method to replace image field with a new resource."""
        request = REQUEST if REQUEST is not None else self.REQUEST
        response = RESPONSE if RESPONSE is not None else request.response

        infile = request.get("BODYFILE", None)
        first_line = infile.readline()
        infile.seek(0)
        if not headerRE.match(safe_text(first_line)):
            self.dav__init(request, response)
            self.dav__simpleifhandler(request, response, refresh=1)

            infile = request.get("BODYFILE", None)
            filename = request["PATH_INFO"].split("/")[-1]
            self.image = NamedBlobImage(data=infile.read(), filename=str(filename))

            modified(self)
            return response
        else:
            return super().PUT(REQUEST=request, RESPONSE=response)

    @security.protected(permissions.View)
    def get_size(self):
        return getattr(self.image, "size", 0)

    @security.protected(permissions.View)
    def content_type(self):
        return getattr(self.image, "contentType", None)


@implementer(ILink)
class Link(Item):
    """Convenience subclass for ``Link`` portal type"""


@implementer(INewsItem)
class NewsItem(Item):
    """Convenience subclass for ``News Item`` portal type"""


@implementer(IEvent)
class Event(Item):
    """Convenience subclass for ``Event`` portal type"""
